@@ -18,33 +18,39 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- ***************************************************************************/ 
+ ***************************************************************************/
 #include "libupnpp/config.h"
 
 #include <stdio.h>
 #include <string>
 #include <sys/types.h>
 
+#ifndef _WIN32
+#include <netinet/in.h>
+#endif
+
 #include <curl/curl.h>
 
 #include "libupnpp/log.hxx"
 #include "libupnpp/control/httpdownload.hxx"
 
 using namespace std;
-using namespace UPnPP;
 
 static size_t
 write_callback(void *contents, size_t size, size_t nmemb, void *userp)
 {
-  size_t realsize = size * nmemb;
-  string* out = (string*)userp;
+    size_t realsize = size * nmemb;
+    string* out = (string*)userp;
 
-  out->append((const char *)contents, realsize);
-  return realsize;
+    out->append((const char *)contents, realsize);
+    return realsize;
 }
 
 
-bool downloadUrlWithCurl(const string& url, string& out, long timeoutsecs)
+namespace UPnPClient {
+
+bool downloadUrlWithCurl(const string& url, string& out, long timeoutsecs,
+    struct sockaddr_storage *saddr)
 {
     CURL *curl;
     CURLcode res;
@@ -55,15 +61,23 @@ bool downloadUrlWithCurl(const string& url, string& out, long timeoutsecs)
         LOGERR("downloadUrlWithCurl: curl_easy_init failed" << endl);
         return false;
     }
-
+    long scopeid{-1};
+    if (nullptr != saddr && saddr->ss_family == AF_INET6) {
+        struct sockaddr_in6 *sa6p = (struct sockaddr_in6 *)saddr;
+        scopeid = (long)sa6p->sin6_scope_id;
+    }
+        
     curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
     curl_easy_setopt(curl, CURLOPT_TIMEOUT, timeoutsecs);
-    curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1); 
+    curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
     curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_callback);
-    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &out); 
+    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &out);
+    if (scopeid != -1) {
+        curl_easy_setopt(curl, CURLOPT_ADDRESS_SCOPE, scopeid);
+    }
     res = curl_easy_perform(curl);
     if(res != CURLE_OK) {
-        LOGERR("downloadUrlWithCurl: curl_easy_perform(): " << 
+        LOGERR("downloadUrlWithCurl: curl_easy_perform(): " <<
                curl_easy_strerror(res) << endl);
     } else {
         ret = true;
@@ -72,3 +86,5 @@ bool downloadUrlWithCurl(const string& url, string& out, long timeoutsecs)
 
     return ret;
 }
+
+}
