@@ -41,6 +41,12 @@
 #include "libupnpp/upnpp_p.hxx"
 #include "vdir.hxx"
 
+#ifndef NPUPNP_AT_LEAST
+#define NPUPNP_AT_LEAST(MAJ,MIN,REV)                                  \
+    VERSION_AT_LEAST(NPUPNP_VERSION_MAJOR,NPUPNP_VERSION_MINOR,       \
+                     NPUPNP_VERSION_PATCH, (MAJ),(MIN),(REV))
+#endif
+
 using namespace std;
 using namespace UPnPP;
 
@@ -71,6 +77,10 @@ public:
     UPnPP::LibUPnP *lib{nullptr};
     string deviceId;
 
+    // For SERVER headers:
+    string product;
+    string version;
+    
     // In case startloop has been called: the event loop thread.
     std::thread loopthread;
 
@@ -208,18 +218,29 @@ UpnpDevice::UpnpDevice(UpnpDevice* rootdev, const string& deviceId)
 
 UpnpDevice::~UpnpDevice()
 {
-    shouldExit();
-    if (m->loopthread.joinable())
-        m->loopthread.join();
+    m->needExit = true;
+    m->evloopcond.notify_all();
 
-    if (nullptr != m->rootdev) {
+    if (nullptr == m->rootdev) {
         UpnpUnRegisterRootDevice(m->dvh);
     }
 
+    if (m->loopthread.joinable())
+        m->loopthread.join();
+
     std::unique_lock<std::mutex> lock(o->devices_lock);
     auto it = o->devices.find(m->deviceId);
     if (it != o->devices.end())
         o->devices.erase(it);
+    delete m;
+}
+
+void UpnpDevice::setProductVersion(const char *product, const char *version)
+{
+    if (product && version) {
+        m->product = product;
+        m->version = version;
+    }
 }
 
 const string& UpnpDevice::getDeviceId() const
@@ -314,6 +335,11 @@ bool UpnpDevice::Internal::start()
         LOGERR("UpnpDevice: libupnp can't start service. Err " << ret << endl);
         return false;
     }
+#endif
+#if NPUPNP_AT_LEAST(4,1,0)
+    if (!product.empty()) {
+        UpnpDeviceSetProduct(dvh, product.c_str(), version.c_str());
+    }
 #endif
     if ((ret = UpnpSendAdvertisement(dvh, expiretime)) != 0) {
         LOGERR("UpnpDevice::Internal::start(): sendAvertisement failed: " <<
@@ -757,6 +783,9 @@ void UpnpDevice::loopWakeup()
 
 void UpnpDevice::shouldExit()
 {
+    if (nullptr == m->rootdev) {
+        UpnpUnRegisterRootDevice(m->dvh);
+    }
     m->needExit = true;
     m->evloopcond.notify_all();
 }
