@@ -62,7 +62,9 @@ struct UPnPOptions {
     std::string ifnames;
     std::string ipv4;
     int port;
-    
+    int substimeout{1800};
+    std::string clientproduct;
+    std::string clientversion;
 };
 static UPnPOptions options;
 
@@ -92,6 +94,15 @@ bool LibUPnP::init(unsigned int flags, ...)
         case UPNPPINIT_OPTION_PORT:
             options.port = va_arg(ap, int);
             break;
+        case UPNPPINIT_OPTION_SUBSCRIPTION_TIMEOUT:
+            options.substimeout = va_arg(ap, int);
+            break;
+        case UPNPPINIT_OPTION_CLIENT_PRODUCT:
+            options.clientproduct = *((std::string*)(va_arg(ap, std::string*)));
+            break;
+        case UPNPPINIT_OPTION_CLIENT_VERSION:
+            options.clientversion = *((std::string*)(va_arg(ap, std::string*)));
+            break;
         default:
             std::cerr << "LibUPnP::init: unknown option value " << option <<"\n";
         }
@@ -186,6 +197,11 @@ std::string LibUPnP::hwaddr()
     return addr;
 }
 
+int LibUPnP::Internal::getSubsTimeout()
+{
+    return options.substimeout;
+}
+
 LibUPnP::LibUPnP()
 {
     bool serveronly = 0 != (options.flags&UPNPPINIT_FLAG_SERVERONLY);
@@ -224,9 +240,14 @@ LibUPnP::LibUPnP()
     }
     setMaxContentLength(2000*1024);
 
+#ifdef UPNP_ENABLE_IPV6
     LOGINF("LibUPnP: Using IPV4 " << UpnpGetServerIpAddress() << " port " <<
            UpnpGetServerPort() << " IPV6 " << UpnpGetServerIp6Address() <<
            " port " << UpnpGetServerPort6() << endl);
+#else
+    LOGINF("LibUPnP: Using IPV4 " << UpnpGetServerIpAddress() << " port " <<
+           UpnpGetServerPort() << endl);
+#endif
 
     // Client initialization is simple, just do it. Defer device
     // initialization because it's more complicated.
@@ -236,6 +257,12 @@ LibUPnP::LibUPnP()
         m->init_error = UpnpRegisterClient(o_callback, (void *)this, &m->clh);
 
         if (m->init_error == UPNP_E_SUCCESS) {
+#if NPUPNP_VERSION >= 40100
+            if (!options.clientproduct.empty()&&!options.clientversion.empty()) {
+                UpnpClientSetProduct(m->clh, options.clientproduct.c_str(),
+                                     options.clientversion.c_str());
+            }
+#endif
             m->ok = true;
         } else {
             LOGERR(errAsString("UpnpRegisterClient", m->init_error) << endl);
